use diesel::result::Error;
use rocket::http::Status;
use rocket::response::status;
use rocket_contrib::json::Json;
use rocket_contrib::templates::Template;
use std::collections::HashMap;

use crate::connection::DbConn;
use crate::snippet;
use crate::snippet::{InsertableSnippet, Snippet};

fn error_response(error: Error) -> Status {
    match error {
        Error::NotFound => Status::NotFound,
        _ => Status::InternalServerError,
    }
}

#[get("/")]
pub fn index() -> Template {
    let context: HashMap<&str, String> = HashMap::new();
    Template::render("index", &context)
}

#[get("/snippets/<id>")]
pub fn show_snippet(id: i32, connection: DbConn) -> Template {
    let result = match snippet::get(&connection, id) {
        Ok(snippet) => Some(snippet),
        Err(_) => None,
    };

    let (title, body) = result.map_or(
        (
            String::from("404 - Snippet not found"),
            format!("<h3'>{}</h3>", "Snippet not found"),
        ),
        |snippet| (snippet.title, snippet.formatted_body),
    );

    let mut context: HashMap<&str, String> = HashMap::new();
    context.insert("title", title);
    context.insert("body", body);

    Template::render("snippets/show", &context)
}

#[post("/api/snippets", format = "application/json", data = "<snippet>")]
pub fn create_snippet<'a>(
    snippet: Json<InsertableSnippet>,
    connection: DbConn,
) -> Result<status::Created<Json<Snippet>>, Status> {
    snippet::insert(snippet.into_inner(), &connection)
        .map(|snippet| status::Created(String::from(""), Some(Json(snippet))))
        .map_err(|error| error_response(error))
}
